import { classes } from '@automapper/classes';
import { AutomapperModule } from '@automapper/nestjs';
import { Test, TestingModule } from '@nestjs/testing';
import { ConfigurationMessager } from '../../adapters/secondaries/configuration.messager';
import { ConfigurationRepository } from '../../adapters/secondaries/configuration.repository';
import { LoggingMessager } from '../../adapters/secondaries/logging.messager';
import { CreateConfigurationCommand } from '../../commands/create-configuration.command';
import { CreateConfigurationRequest } from '../../domain/dtos/create-configuration.request';
import { Domain } from '../../domain/dtos/domain.enum';
import { Configuration } from '../../domain/entities/configuration';
import { CreateConfigurationUseCase } from '../../domain/usecases/create-configuration.usecase';
import { ConfigurationProfile } from '../../mappers/configuration.profile';

const newConfigurationRequest: CreateConfigurationRequest = {
  domain: Domain.USER,
  key: 'minAge',
  value: '16',
};
const newConfigurationCommand: CreateConfigurationCommand =
  new CreateConfigurationCommand(newConfigurationRequest);

const mockConfigurationRepository = {
  create: jest
    .fn()
    .mockImplementationOnce(() => {
      return Promise.resolve({
        ...newConfigurationRequest,
        uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
      });
    })
    .mockImplementation(() => {
      throw new Error('Already exists');
    }),
};

const mockMessager = {
  publish: jest.fn().mockImplementation(),
};

describe('CreateConfigurationUseCase', () => {
  let createConfigurationUseCase: CreateConfigurationUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [AutomapperModule.forRoot({ strategyInitializer: classes() })],
      providers: [
        {
          provide: ConfigurationRepository,
          useValue: mockConfigurationRepository,
        },
        CreateConfigurationUseCase,
        ConfigurationProfile,
        {
          provide: ConfigurationMessager,
          useValue: mockMessager,
        },
        {
          provide: LoggingMessager,
          useValue: mockMessager,
        },
      ],
    }).compile();

    createConfigurationUseCase = module.get<CreateConfigurationUseCase>(
      CreateConfigurationUseCase,
    );
  });

  it('should be defined', () => {
    expect(createConfigurationUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should create and return a new configuration', async () => {
      const newConfiguration: Configuration =
        await createConfigurationUseCase.execute(newConfigurationCommand);

      expect(newConfiguration.key).toBe(newConfigurationRequest.key);
      expect(newConfiguration.uuid).toBeDefined();
    });

    it('should throw an error if configuration already exists', async () => {
      await expect(
        createConfigurationUseCase.execute(newConfigurationCommand),
      ).rejects.toBeInstanceOf(Error);
    });
  });
});
